import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Ch103FileLoader {

    public Ch103FileLoader(String filename) {
        List<String> contents = readFile(filename);
        contents.forEach(System.out::println);

        contents = readFile2(filename);
        contents.forEach(System.out::println);
    }

    // リスト1.29 ファイルの読み込み
    List<String> readFile(String filename) {
        BufferedReader reader = null;
        List<String> contents = new ArrayList<>();

        try {
            reader = new BufferedReader(new FileReader(filename));

            // 1行読み込んでcontentsに追加する
            for (;;) {
                String line = reader.readLine();

                if (line == null) {
                    break;
                } else {
                    contents.add(line);
                }
            }

        } catch (IOException ex) {
            // 例外処理
        } finally {
            // クローズ処理
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException ex) {
                }
            }
        }

        return contents;
    }

    // リスト1.31 try-with-resources構文を使用したファイルの読み込み
    private List<String> readFile2(String filename) {
        List<String> contents = new ArrayList<>();

        try (BufferedReader reader = new BufferedReader(new FileReader(filename))) {
            // 1行読み込んでcontentsに追加する
            for (;;) {
                String line = reader.readLine();

                if (line == null) {
                    break;
                } else {
                    contents.add(line);
                }
            }

        } catch (IOException ex) {
            // 例外処理
            Logger.getGlobal().log(Level.SEVERE, "ファイルが開けませんでした", ex);
        }

        return contents;
    }

    public static void main(String... args) {
        if (args.length > 0) {
            new Ch103FileLoader(args[0]);
        }
    }
}
