import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Ch104FileCopy {

    private static final int BUFFER_SIZE = 1_000;

    public Ch104FileCopy(String srcfile, String destfile) {
        copy(srcfile, destfile);
    }

    // リスト1.32 try-with-resources構文で複数リソースを扱う場合
    private void copy(String srcfile, String destfile) {
        try (BufferedInputStream inStream = new BufferedInputStream(new FileInputStream(srcfile));
                BufferedOutputStream outStream = new BufferedOutputStream(new FileOutputStream(destfile))) {

            byte[] buffer = new byte[BUFFER_SIZE];
            for (;;) {
                // lengthバイト読み込んで、それを書き込む
                int length = inStream.read(buffer);
                if (length != -1) {
                    outStream.write(buffer, 0, length);
                } else {
                    break;
                }
            }
        } catch (IOException ex) {
            // 例外処理
            Logger.getGlobal().log(Level.SEVERE, "ファイルがコピーできませんでした", ex);
        }
    }

    public static void main(String... args) {
        if (args.length > 1) {
            new Ch104FileCopy(args[0], args[1]);
        }
    }
}
