import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.FileTime;
import java.nio.file.attribute.PosixFilePermission;
import java.nio.file.attribute.PosixFilePermissions;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Ch222SimpleAttributeReader {

    public Ch222SimpleAttributeReader() {
        try {
            readAttribute();
        } catch (IOException ex) {
            Logger.getGlobal().log(Level.SEVERE, "Fail dealing Attribute", ex);
        }
    }

    private void readAttribute() throws IOException {
        // リスト2.32 Filesクラスのアトリビュート取得メソッドの使用例

        // 対象となるパス
        Path path = Paths.get("foo.txt");

        // 最終変更日時を取得
        FileTime time = Files.getLastModifiedTime(path);
        System.out.println("Last Modify: " + time);

        if (!System.getProperty("os.name").contains("Windows")) {
            // パーミッションを取得 (Windowsは不可)
            Set<PosixFilePermission> permissions = Files.getPosixFilePermissions(path);
            // パーミッションをUNIXの文字列表記に変換する
            System.out.println("Permissions: " + PosixFilePermissions.toString(permissions));
        } else {
            System.out.println("Executable: " + Files.isExecutable(path));
            System.out.println("Readable: " + Files.isReadable(path));
            System.out.println("Writable: " + Files.isWritable(path));
        }
    }

    public static void main(String... args) {
        new Ch222SimpleAttributeReader();
    }
}
