import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.DosFileAttributeView;
import java.nio.file.attribute.DosFileAttributes;
import java.nio.file.attribute.PosixFileAttributeView;
import java.nio.file.attribute.PosixFileAttributes;
import java.nio.file.attribute.PosixFilePermissions;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Ch226AttributesReader {

    public Ch226AttributesReader() {
        try {
            readAttributes();
        } catch (IOException ex) {
            Logger.getGlobal().log(Level.SEVERE, "Fail getting Attribute", ex);
        }
    }

    private void readAttributes() throws IOException {
        // 対象となるパス
        Path path = Paths.get("foo.txt");

        if (System.getProperty("os.name").contains("Windows")) {
            // リスト2.37 Windowsで複数のアトリビュートをまとめて取得

            // Windows用のDosFileAttributeViewオブジェクトを取得
            DosFileAttributeView view = Files.getFileAttributeView(path, DosFileAttributeView.class);

            // Windows用のDosFileAttributesオブジェクトの取得
            DosFileAttributes attributes = view.readAttributes();

            // BasicFileAttributesインタフェースで定義されたメソッド
            System.out.println("Creation Time: " + attributes.creationTime());
            System.out.println("Last Modified Time: " + attributes.lastModifiedTime());
            System.out.println("Last Access Time: " + attributes.lastAccessTime());
            System.out.println("File:  " + attributes.isRegularFile());
            System.out.println("Directory: " + attributes.isDirectory());
            System.out.println("Symbolic Link: " + attributes.isSymbolicLink());

            // DosFileAttributesインタフェースで定義されたメソッド
            System.out.println("System File: " + attributes.isSystem());
        } else {
            // リスト2.38 Posix準拠OSで複数のアトリビュートをまとめて取得

            // Posix準拠OS用のPosixFileAttributeViewオブジェクトを取得
            PosixFileAttributeView view = Files.getFileAttributeView(path, PosixFileAttributeView.class);

            // Posix準拠OS用のPosixFileAttributesオブジェクトの取得
            PosixFileAttributes attributes = view.readAttributes();

            // BasicFileAttributesインタフェースで定義されたメソッド
            System.out.println("Creation Time: " + attributes.creationTime());
            System.out.println("Last Modified Time; " + attributes.lastModifiedTime());
            System.out.println("Last Access Time: " + attributes.lastAccessTime());
            System.out.println("File:  " + attributes.isRegularFile());
            System.out.println("Directory: " + attributes.isDirectory());
            System.out.println("Symbolic Link: " + attributes.isSymbolicLink());

            // PosixFileAttributesインタフェースで定義されたメソッド
            System.out.println("Owner: " + attributes.owner());
            System.out.println("Permision: " + PosixFilePermissions.toString(attributes.permissions()));
        }
    }

    public static void main(String... args) {
        new Ch226AttributesReader();
    }
}
