import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.DosFileAttributeView;
import java.nio.file.attribute.FileTime;
import java.nio.file.attribute.PosixFileAttributeView;
import java.nio.file.attribute.UserPrincipal;
import java.nio.file.attribute.UserPrincipalLookupService;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Ch227AttributesWriter {

    public Ch227AttributesWriter() {
        try {
            writeAttributes();
        } catch (IOException ex) {
            Logger.getGlobal().log(Level.SEVERE, "Fail writing Attribute", ex);
        }
    }

    private void writeAttributes() throws IOException {

        // 対象となるパス
        Path path = Paths.get("foo.txt");

        if (System.getProperty("os.name").contains("Windows")) {
            // リスト2.39 Windowsで複数のアトリビュートをまとめて設定

            // Windows用のDosFileAttributeViewオブジェクトを取得
            DosFileAttributeView view = Files.getFileAttributeView(path, DosFileAttributeView.class);

            // BasicFileAttributeViewインタフェースで定義されたメソッド
            long now = System.currentTimeMillis();
            view.setTimes(FileTime.fromMillis(now), // 最終更新日時
                          FileTime.fromMillis(now), // 最終アクセス日時
                          FileTime.fromMillis(now)); // 生成日時

            // DosFileAttributeViewインタフェースで定義されたメソッド
            // システムファイルとして設定する
            view.setSystem(true);
        } else {
            // リスト2.40 Posix準拠OSで複数のアトリビュートをまとめて設定

            // Posix準拠OS用のPosixFileAttributeViewオブジェクトを取得
            PosixFileAttributeView view = Files.getFileAttributeView(path, PosixFileAttributeView.class);

            // BasicFileAttributeViewインタフェースで定義されたメソッド
            long now = System.currentTimeMillis();
            view.setTimes(FileTime.fromMillis(now), // 最終更新日時
                          FileTime.fromMillis(now), // 最終アクセス日時
                          FileTime.fromMillis(now)); // 生成日時

            // アカウントの情報を取得
            UserPrincipalLookupService lookupService
                = FileSystems.getDefault().getUserPrincipalLookupService();
            UserPrincipal owner = lookupService.lookupPrincipalByName("sakuraba");
            // PosixFileAttributeViewインタフェースで定義されたメソッド
            view.setOwner(owner);
        }
    }

    public static void main(String... args) {
        new Ch227AttributesWriter();
    }
}
