import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardWatchEventKinds;
import java.nio.file.WatchEvent;
import java.nio.file.WatchKey;
import java.nio.file.WatchService;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Ch230DirecotryWatch {

    public Ch230DirecotryWatch(String targetDirectory) {
        try {
            watchDirectory(targetDirectory);
        } catch (IOException ex) {
            Logger.getLogger(Ch230DirecotryWatch.class.getName()).log(Level.SEVERE, "Failed Directory Watch", ex);
        }
    }

    private void watchDirectory(String targetDirectory) throws IOException {

        // リスト2.44 WatchServiceオブジェクトの取得と監視の登録

        // WatchServiceオブジェクトの生成
        WatchService watchService = FileSystems.getDefault().newWatchService();

        // 監視の対象となるディレクトリ
        Path target = Paths.get(targetDirectory);

        // ディレクトリの監視を登録
        target.register(watchService,
                StandardWatchEventKinds.ENTRY_CREATE,
                StandardWatchEventKinds.ENTRY_DELETE,
                StandardWatchEventKinds.ENTRY_MODIFY);

        // リスト2.45 ディレクトリ監視のポーリング
        while (true) {
            try {
                // 現在発生しているキーを取得
                WatchKey key = watchService.take();

                // キーからイベントを取得
                for (WatchEvent<?> event : key.pollEvents()) {
                    if (event.kind() == StandardWatchEventKinds.OVERFLOW) {
                        // イベントがオーバフローしてしまった場合
                        key.reset();
                        continue;
                    } else {
                        // それ以外の場合、
                        // イベントの種類と対象となるパスを出力
                        System.out.println(event.kind() + " " + event.context());
                    }

                    // キーをリセット
                    if (!key.reset()) {
                        break;
                    }
                }
            } catch (InterruptedException ex) {
                break;
            }
        }

        // リスト2.46 ディレクトリ監視の終了
        watchService.close();
    }

    public static void main(String... args) {
        if (args.length > 0) {
            new Ch230DirecotryWatch(args[0]);
        }
    }
}
