import java.util.function.Function;

public class Ch311FunctionSynthesizeDemo {
    // リスト3.72 Pointクラス
    static class Point {
        double x;
        double y;

        private Point(double x, double y) {
            this.x = x;
            this.y = y;
        }

        public static Point of(double x, double y) {
            return new Point(x, y);
        }

        public Point transform(Function<Point, Point> function) {
            return function.apply(this);
        }

        public String toString() {
            return "[" + x + ", " + y + "]";
        }
    }

    // リスト3.73 移動と回転を行う関数を生成するメソッド

    private Function<Point, Point> createMoveFunction(double dx, double dy) {
        return p -> Point.of(p.x + dx, p.y + dy);
    }

    private Function<Point, Point> createRotateFunction(double rad) {
        return p -> Point.of(p.x*Math.cos(rad) - p.y*Math.sin(rad),
                             p.x*Math.sin(rad) + p.y*Math.cos(rad));
    }

    public Ch311FunctionSynthesizeDemo() {

        // リスト3.74 アフィン変換の実行例

        // 変換を行う対象となる点
        Point point = Point.of(5, 5);

        // 移動を行う関数
        Function<Point, Point> move = createMoveFunction(5, 0);
        // 回転を行う関数
        Function<Point, Point> rotate = createRotateFunction(90.0*Math.PI/180.0);

        // 移動を行ってから、回転を行う関数
        Function<Point, Point> trans1 = move.andThen(rotate);
        Point newPoint1 = point.transform(trans1);

        // 回転を行ってから、移動を行う関数
        Function<Point, Point> trans2 = rotate.andThen(move);
        Point newPoint2 = point.transform(trans2);

        System.out.println(newPoint1);
        System.out.println(newPoint2);
    }

    public static void main(String... args) {
        new Ch311FunctionSynthesizeDemo();
    }
}
