import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.IntSummaryStatistics;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class Ch404ForToStream {
    private List<Customer> customers;
    private List<Item> items;

    public Ch404ForToStream() {
        initData();
    }

    private void initData() {
        items = new ArrayList<>();
        Item item1 = Item.of(1, "A101", 10_000);
        items.add(item1);
        Item item2 = Item.of(2, "G-303", 25_000);
        items.add(item2);
        Item item3 = Item.of(3, "Z 11", 20_000);
        items.add(item3);
        Item item4 = Item.of(4, "B 222", 5_000);
        items.add(item4);
        Item item5 = Item.of(5, "J1-05", 8_000);
        items.add(item5);

        customers = new ArrayList<>();

        Customer customer1 = Customer.of(1, "Miles Davis", "Illinois", item1, item4);
        customers.add(customer1);
        Customer customer2 = Customer.of(2, "Paul Chambers", "Pennsylvania", item2, item3, item4);
        customers.add(customer2);
        Customer customer3 = Customer.of(3, "Milt Jackson", "Michigan", item1, item3);
        customers.add(customer3);
        Customer customer4 = Customer.of(4, "Red Garland", "Texas", item1, item4, item5);
        customers.add(customer4);
        Customer customer5 = Customer.of(5, "John Coltrane", "North Carolina", item1, item2, item4);
        customers.add(customer5);
        Customer customer6 = Customer.of(6, "Thelonious Monk", "North Carolina", item3, item4, item5);
        customers.add(customer6);
        Customer customer7 = Customer.of(7, "McCoy Tyner", "Pennsylvania", item2, item5);
        customers.add(customer7);
        Customer customer8 = Customer.of(8, "Dizzy Gillespie", "South Carolina", item3);
        customers.add(customer8);
        Customer customer9 = Customer.of(9, "Charles Parker", "Kansas", item1, item3);
        customers.add(customer9);
        Customer customer10 = Customer.of(10, "Herbie Hancock", "Illinois", item2, item4, item5);
        customers.add(customer10);
    }

    public void transform() {
        // 指定した製品を購入した人を抽出
        List<Customer> purchaser1 = expressCustomerByFor(items.get(0));
        List<Customer> purchaser2 = expressCustomerByStream(items.get(0));
        System.out.println("expressCustomer: " + purchaser1.equals(purchaser2));

        // 名前の一覧を取得し、ソート
        List<String> names1 = getCustomerNamesByFor();
        List<String> names2 = getCustomerNamesByFor();
        System.out.println("getCustomerNames: " + names1.equals(names2));

        // 売れた製品の一覧
        List<Item> items1 = getAllItemByFor();
        List<Item> items2 = getAllItemByStream();
        System.out.println("getAllItem: " + items1.equals(items2));

        Map<String, List<Customer>> customerMap1 = groupCostomerByFor();
        Map<String, List<Customer>> customerMap2 = groupCostomerByStream();
        System.out.println("groupeCustomer: " + customerMap1.equals(customerMap2));

        // 製品と売れた個数のマップ
        Map<Item, Long> itemCount1 = getItemCountByFor();
        Map<Item, Long> itemCount2 = getItemCountByStream();
        System.out.println("getItemCount: " + itemCount1.equals(itemCount2));

        Map<Customer, IntSummaryStatistics> customerStats1 = getCostomerStatisticsByFor();
        Map<Customer, IntSummaryStatistics> customerStats2 = getCostomerStatisticsByStream();
        boolean result = true;
        for (Customer c: customerStats1.keySet()) {
            IntSummaryStatistics stats1 = customerStats1.get(c);
            IntSummaryStatistics stats2 = customerStats2.get(c);
            result &= stats1.getCount() == stats2.getCount()
                    && stats1.getMin() == stats2.getMin()
                    && stats1.getMax() == stats2.getMax()
                    && stats1.getSum()== stats2.getSum()
                    && stats1.getAverage()== stats2.getAverage();
        }
        System.out.println("getCustomerStatistics: " + result);
    }

    // リスト4.95 指定した製品を購入した顧客を抽出
    public List<Customer> expressCustomerByFor(Item item) {
        List<Customer> purchaser = new ArrayList<>();

        for (Customer customer: customers) {
            if (customer.getHistory().contains(item)) {
                purchaser.add(customer);
            }
        }

        return purchaser;
    }

    // リスト4.97 collectメソッドを使用して、指定した製品を購入した顧客を抽出
    public List<Customer> expressCustomerByStream(Item item) {
        return customers.stream()
                        .filter(customer -> customer.getHistory().contains(item))
                        .collect(Collectors.toList());
    }

    // リスト4.98 顧客リストから顧客の名前一覧を取りだす
    public List<String> getCustomerNamesByFor() {
        List<String> names = new ArrayList<>();

        for (Customer customer: customers) {
            String name = customer.getName();
            names.add(name);
        }

        names.sort(Comparator.naturalOrder());

        return names;
    }

    // リスト4.102 ソートした顧客の名前一覧を取りだす
    public List<String> getCustomerNamesByStream() {
        return customers.stream()
                        .map(customer -> customer.getName())
                        .sorted()
                        .collect(Collectors.toList());
    }

    public List<Item> getAllItemByFor() {
        List<Item> items = new ArrayList<>();

        for (Customer customer: customers) {
            List<Item> history = customer.getHistory();

            for (Item item: history) {
                if (!items.contains(item)) {
                    items.add(item);
                }
            }
        }

        items.sort(new Comparator<Item>() {
            @Override
            public int compare(Item item1, Item item2) {
                return Integer.compare(item1.getId(), item2.getId());
            }
        });

        return items;
    }

    // リスト4.108 Stream APIを使用して、購入された製品の一覧を抽出
    public List<Item> getAllItemByStream() {
        return customers.stream()
                        .flatMap(customer -> customer.getHistory().stream())
                        .distinct()
                        .sorted((item1, item2) -> Integer.compare(item1.getId(), item2.getId()))
                        .collect(Collectors.toList());
    }

    // リスト4.109 地域によって顧客をグルーピング
    public Map<String, List<Customer>> groupCostomerByFor() {
        Map<String, List<Customer>> customerMap = new HashMap<>();

        for (Customer customer: customers) {
            String area = customer.getArea();

            List<Customer> tempCustomers = customerMap.getOrDefault(area, new ArrayList<>());
            tempCustomers.add(customer);
            customerMap.put(area, tempCustomers);
        }

        return customerMap;
    }

    // リスト4.110 Stream APIを使用して、地域による顧客のグルーピング
    public Map<String, List<Customer>> groupCostomerByStream() {
        Map<String, List<Customer>> customerMap
            = customers.stream()
                       .collect(Collectors.groupingBy(customer -> customer.getArea()));

        return customerMap;
    }


    // リスト4.112 製品ごとの販売数を集計
    public Map<Item, Long> getItemCountByFor() {
        Map<Item, Long> itemCounts = new HashMap<>();

        for (Customer customer: customers) {
            List<Item> history = customer.getHistory();
            for (Item item: history) {
                long count = itemCounts.getOrDefault(item, 0L);
                count++;
                itemCounts.put(item, count);
            }
        }

        return itemCounts;
    }

    // リスト4.114 Stream APIを使用して、製品ごとの販売数を集計
    public Map<Item, Long> getItemCountByStream() {
        Map<Item, Long> itemCounts
            = customers.stream()
                       .flatMap(customer -> customer.getHistory().stream())
                       .collect(Collectors.groupingBy(item -> item,
                                                      Collectors.counting()));

        return itemCounts;
    }

    // リスト4.115 顧客ごとの購買履歴の統計をもとめる
    public Map<Customer, IntSummaryStatistics> getCostomerStatisticsByFor() {
        Map<Customer, IntSummaryStatistics> customerStats = new HashMap<>();

        for (Customer customer: customers) {
            List<Item> history = customer.getHistory();
            IntSummaryStatistics stats = new IntSummaryStatistics();

            for (Item item: history) {
                stats.accept(item.getPrice());
            }

            customerStats.put(customer, stats);
        }

        return customerStats;
    }

    // リスト4.117 Stream APIを使用して、顧客ごとの購買履歴の統計をもとめる
    public Map<Customer, IntSummaryStatistics> getCostomerStatisticsByStream() {
        Map<Customer, IntSummaryStatistics> customerStats
            = customers.stream()
                       .collect(Collectors.toMap(
                           customer -> customer,
                           customer -> {
                               return customer.getHistory()
                                              .stream()
                                              .collect(Collectors.summarizingInt(
                                                  item -> item.getPrice()));
                           }));

        return customerStats;
    }

    public static void main(String... args) {
        Ch404ForToStream sample = new Ch404ForToStream();

        sample.transform();
    }
}
