import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveTask;

// リスト4.129 Fork/Join Frameworkで合計を求めるタスク
class Adder extends RecursiveTask<Integer> {
    private final int[] array;
    private final int start;
    private final int end;

    Adder(int[] array, int start, int end) {
        this.array = array;
        this.start = start;
        this.end = end;
    }

    @Override
    public Integer compute() {
        // 分割が1まで達したら、要素をそのまま返す
        if (start == end) {
            return array[start];
        }

        // 分割するインデックスを求める
        int middle = (start + end) / 2;

        // 分割した部分に対して合計処理を行うタスクを生成
        Adder lTask = new Adder(array, start, middle);
        Adder rTask = new Adder(array, middle + 1, end);

        // タスクの一方はそのまま処理を行う
        int left = lTask.compute();

        // もう一方のタスクをフォークし、パラレルに実行する
        rTask.fork();

        // パラレルに実行した結果を受け取る
        int right = rTask.join();

        // 結果を統合し、返り値とする
        return left + right;
    }
}

public class Ch405ForkJoinFrameworkDemo {

    public void sum() {
        // リスト4.130 Fork/Join Frameworkでのタスク実行

        // 計算対象となる配列
        int[] array = {3, 8, 1, 2, 9, 5, 0, 7};

        // ForkJoin Frameworkで実行するためのスレッドプールを取得
        ForkJoinPool pool = ForkJoinPool.commonPool();

        // タスクを生成
        Adder task = new Adder(array, 0, array.length - 1);

        // タスクをスレッドプールに登録
        int total = pool.invoke(task);
        System.out.println(total);
    }

    public static void main(String... args) {
        Ch405ForkJoinFrameworkDemo demo = new Ch405ForkJoinFrameworkDemo();
        demo.sum();
    }
}
